﻿Public Class btnLoadPagesSynchronous
    Const msdnURL = "http://www.msdn.com"
    Const bingURL = "http://www.bing.com"

#Region "Synchronous"

    'Const msdnURL = "http://www.msdn.com"
    'Const bingURL = "http://www.bing.com"

    Private Sub btnLoadPagesSync_Click(sender As Object, e As EventArgs) Handles btnLoadPagesSync.Click
        'No web pages will appear until they are all loaded
        Call DoLoadPagesSync()
    End Sub

    Private Sub DoLoadPagesSync()
        'Load each page synchronously over http for demo
        btnLoadPagesSync.BackColor = Color.Red
        Application.DoEvents()
        Dim StartTime = Now
        UpdateWebControlSync(msdnURL, wb1)
        UpdateWebControlSync(bingURL, wb2)
        UpdateWebControlSync(msdnURL, wb3)
        UpdateWebControlSync(bingURL, wb4)
        btnLoadPagesSync.BackColor = System.Windows.Forms.Button.DefaultBackColor
        lblElapsedSynch.Text = "Button response Time: " & DateTime.Now.Subtract(StartTime).ToString
    End Sub

    Private Sub UpdateWebControlSync(webPageURL As String, wc As WebBrowser)
        'Simple method to display web pages on a windows form
        Try
            wc.DocumentText = ""

            'Using synchonous Get method rather than asynchronous GetAsync() method
            'Synchronous call to retrieve the HTML, method is not suspended during load
            Dim response = New System.Net.Http.HttpClient().Get(New System.Uri(webPageURL))

            'Sync method as completed and execution continues
            Dim pageHTML = response.Content.ReadAsString
            wc.Document.Write(pageHTML)
        Catch ex As Exception
            Dim msg = "Exception: {0}" & vbCrLf & vbCrLf & "{1}"
            MessageBox.Show(String.Format(msg, ex.Message, ex.InnerException.Message))
        End Try
        Return
    End Sub


#End Region

#Region "Asynchronous"

    'Const msdnURL = "http://www.msdn.com"
    'Const bingURL = "http://www.bing.com"

    Private Sub btnLoadPageAsync_Click(sender As Object, e As EventArgs) Handles btnLoadPagesAsync.Click
        'Longer load times for msdnURL (pages 1 and 3), will see bingURL on pages 2 and 4 load first
        Call DoLoadPagesAsync()
    End Sub

    Private Sub DoLoadPagesAsync()
        'Load each page ansynchronously over http for demo
        btnLoadPagesAsync.BackColor = Color.Red
        Dim StartTime = Now
        UpdateWebControlAsync(msdnURL, wb1)
        UpdateWebControlAsync(bingURL, wb2)
        UpdateWebControlAsync(msdnURL, wb3)
        UpdateWebControlAsync(bingURL, wb4)
        btnLoadPagesAsync.BackColor = System.Windows.Forms.Button.DefaultBackColor
        lblElapsedAsync.Text = "Button Response Time: " & DateTime.Now.Subtract(StartTime).ToString
    End Sub

    Private Async Sub UpdateWebControlAsync(webPageURL As String, wc As WebBrowser)
        'Simple method to display web pages on a windows form
        'Note Await inside Try block
        Try
            wc.DocumentText = ""

            'Using asynchrounous GetAsync method rather than synchronous Get() method
            'Execution is suspended by Await modifier until GetAsync() returns
            Dim response = Await New System.Net.Http.HttpClient().GetAsync(New System.Uri(webPageURL))

            'Asynch method has completed and execution resumed
            Dim pageHTML = response.Content.ReadAsString
            wc.Document.Write(pageHTML)
        Catch ex As Exception
            Dim msg = "Exception: {0}" & vbCrLf & vbCrLf & "{1}"
            MessageBox.Show(String.Format(msg, ex.Message, ex.InnerException.Message))
        End Try
        Return
    End Sub

#End Region

#Region "Incorrect Asynchronous"

    Private Sub btnIncorrectAsync_Click(sender As Object, e As EventArgs) Handles btnIncorrectAsync.Click
        Call IncorrectCallerAsync()
    End Sub

    Private Sub IncorrectCallerAsync()
        'Illustrates a usage pattern for Async and Await that will fail
        '"Method completed:" displays 1-2 seconds before "HTML received:" message
        wb1.DocumentText = ""
        Dim HTMLtask = GetHTMLAsync(msdnURL)
        If (HTMLtask.Status = TaskStatus.RanToCompletion) Then
            'This never executes since task returns TaskStatus.WaitingForActivation
            Dim HTML As String = HTMLtask.Result
            wb1.Document.Write(HTML)
            lblElapsedSynch.Text = "Page 1 Updated: " & Now.TimeOfDay.ToString
        End If
        lblElapsedSynch.Text = "Method completed: " & Now.TimeOfDay.ToString
    End Sub
    Private Async Function GetHTMLAsync(webPageURL As String) As Task(Of String)
        Dim HTMLcontent = String.Empty
        Try
            Dim response = Await New System.Net.Http.HttpClient().GetAsync(New System.Uri(webPageURL))
            HTMLcontent = response.Content.ReadAsString
            lblElapsedAsync.Text = "HTML received: " & Now.TimeOfDay.ToString
        Catch ex As Exception
            Dim msg = "Exception: {0}" & vbCrLf & vbCrLf & "{1}"
            MessageBox.Show(String.Format(msg, ex.Message, ex.InnerException.Message))
        End Try
        Return HTMLcontent
    End Function
#End Region

    Public Sub New()
        InitializeComponent()
        AddHandler btnLambdaAsync.Click, Async Sub(sender, e)
                                             wb2.DocumentText = ""
                                             Dim response = Await New System.Net.Http.HttpClient().GetAsync(New System.Uri(bingURL))
                                             Dim pageHTML = response.Content.ReadAsString
                                             wb2.Document.Write(pageHTML)
                                         End Sub
    End Sub

    Private Async Sub btnButtonAsync_Click(sender As Object, e As EventArgs) Handles btnButtonAsync.Click
        wb2.DocumentText = ""
        Dim response = Await New System.Net.Http.HttpClient().GetAsync(New System.Uri(bingURL))
        Dim pageHTML = response.Content.ReadAsString
        wb2.Document.Write(pageHTML)
    End Sub
End Class
